/*********************************************************************
   Analog Interface Unit (AIU) - Driver Routines
   Verison 1.0
   Written in Borland Turbo C
   Last Updated : 28/5/95

   Copyright (c) 1995 David L. Jones
   Advanced R&D Solutions - TRONNORT Technology
   12 Copeland Road
   Lethbridge Park NSW 2770
   Australia
**********************************************************************/

#include<math.h>;

unsigned int PortAddr;

/*********************************************************************
Description : Returns the Two's Complement form of b
**********************************************************************/
int BinaryToTwos(int b)
{
   b=b ^ 0xff;
   b=b & 0x7f;
   if (b<127) b++;
   b=b | 128;
   return(b);
}


/*********************************************************************
Description : Returns the Binary form of Two's Complement b
**********************************************************************/
int TwosToBinary(int b)
{
   b=b & 0x7f;
   if (b > 0) b--;
   b=b ^ 0xff;
   b=b & 0x7f;
   return(b);
}


/*********************************************************************
Description : Returns the current byte on the AIU's data bus
Parameters : None
**********************************************************************/
int ReadByte(void)
{
   int t;                                    /* final value */
   int v;                                    /* temp port variable */
   t=0;                                       /* reset final value */
   outportb(PortAddr, (inportb(PortAddr) & 0xfb));   /* selet A data - xxxxx0xx */
   v=inportb(PortAddr+1);                           /* get port value */
   v=v ^ 128;                                 /* invert last bit */
   if ((v & 128) > 0) t+=128;
   if ((v & 64)  > 0) t+=64;
   if ((v & 32)  > 0) t+=1;
   if ((v & 16)  > 0) t+=8;

   outport(PortAddr, (inportb(PortAddr) | 4));      /* selet B data - xxxxx1xx */
   v=inportb(PortAddr+1);                           /* get port value */
   v=v ^ 128;                                       /* invert last bit */
   if ((v & 128) > 0) t+=32;
   if ((v & 64)  > 0) t+=16;
   if ((v & 32)  > 0) t+=2;
   if ((v & 16)  > 0) t+=4;
   return(t);
}


/*********************************************************************
Description : Clocks a bit into the AIU
Parameters :
   bit : Bit to be sent to AIU - Boolean
**********************************************************************/
void ClockData(int bit)
{
   if (bit==0) outportb(PortAddr, (inportb(PortAddr) & 0xfb));   /* clear bit */
      else outportb(PortAddr, (inportb(PortAddr) | 4));         /* set bit */
   outportb((PortAddr+2), (inportb(PortAddr+2) & 0xfb));       /* latch clock low  - XXXXX0XX */
   outportb((PortAddr+2), (inportb(PortAddr+2) | 4));              /* latch clock high - XXXXX1XX */
}

/*********************************************************************
Description : Sends a Value to the ADC
Parameters :
   v : Byte to be sent to the ADC
**********************************************************************/
void SetByte(int v)
{
   int retry;
   retry=0;
   do {
      ClockData((v & 1));                           /* set bit 0 */
      ClockData((v & 16));                           /* set bit 4 */
      ClockData((v & 64));                           /* set bit 6 */
      ClockData((v & 128));                        /* set bit 7 */
      ClockData((v & 2));                           /* set bit 1 */
      ClockData((v & 4));                           /* set bit 2 */
      ClockData((v & 8));                           /* set bit 3 */
      ClockData((v & 32));                           /* set bit 5 */
      retry++;
   } while ((v!=ReadByte()) && (retry<100));

   outportb(PortAddr, (inportb(PortAddr) | 1));      /* set wr high - xxxxxxx1 */
   outportb(PortAddr, (inportb(PortAddr) & 0xfe));   /* set wr low  - xxxxxxx0 */
   outportb(PortAddr, (inportb(PortAddr) | 1));      /* set wr high - xxxxxxx1 */
}


/*********************************************************************
Description : Returns the actual 8 bit ADC value
Parameters : None
**********************************************************************/
int ReadABS(void)
{
   long int loop;
   int temp;
   outportb(PortAddr, (inportb(PortAddr) | 1));      /* turn "wr" off     - xxxxxxx1 */
   outportb(PortAddr, (inportb(PortAddr) & 0xfd));   /* set ADC to "read" - xxxxxx0x */
   loop=0;
   do { loop++;   }                       /* wait until end of conversion */
   while (((inportb(PortAddr+1) & 8)!=0) && (loop<100000));
   temp=ReadByte();
   outportb(PortAddr, (inportb(PortAddr) | 2));   /* turn off "read" - xxxxxx1x */
   return(temp);
}


/*********************************************************************
Description : Returns the actual ADC voltage
Parameters :
   Range : 0=+/-1.25v   else =+/-2.5v
**********************************************************************/
float ReadVoltage(int Range)
{
   int b,f,neg;
   float t;
   b=ReadABS();                           /* get the ADC value */

   if ((b & 0x80)!=0) neg=-1; else neg=0;
   if (neg!=0) {
      b=TwosToBinary(b);                  /* convert from 2's complement */
      f=1;                                 /* tiny fudge factor */
      if (Range!=0) t=((float)(b+f)/128.0)*2.5;
         else t=((float)(b+f)/128.0)*1.25;
      }                                   /* calculate voltage */
   else {
      if ((b==0) || (b==127)) f=0; else f=1;         /* fudge factor */
      if (Range!=0) t=((float)(b-f)/128.0)*2.5;
         else t=((float)(b-f)/128.0)*1.25;
      }                                     /* calculate voltage */
   if (neg!=0) t=-t;                        /* turn value into negative */

   return(t);
}


/*********************************************************************
Description : Turns the AIU power on and off
Parameters :
   OnOff : 0=off   <>0=on
**********************************************************************/
void Power(int OnOff)
{
   if (OnOff!=0) outportb(PortAddr,0xff);         /* power on */
      else outportb(PortAddr,0);                  /* power off */
   outportb((PortAddr+2), 0);                     /* disable port interupts */
}


/*********************************************************************
Description : Sets the range of the AIU to +/-2.5v or +/-1.25v
Parameters :
   Range : 0=+/-1.25v   <>0=+/-2.5v
**********************************************************************/
void SetRange(int Range)
{
   if (Range!=0) outportb((PortAddr+2), (inportb(PortAddr+2) & 0xfe)); /* +/- 2.5v */
      else outportb((PortAddr+2), (inportb(PortAddr+2) | 1)); /* +/- 1.25v */
}


/*********************************************************************
Description : Sets the DAC voltage
Parameters :
   r : Range 0=+/-1.25v    else =+/-2.5v
   v : voltage to set DAC at
**********************************************************************/
void SetVoltage(float v, int r)
{
   int d,b,neg;
   float temp;
   if (r!=0) {
      if (v>2.5) v=2.5;              /* clip input voltage range to +/-2.5 */
      if (v<-2.5) v=-2.5;
      }
   else {
      if (v>1.25) v=1.25;             /* clip input voltage range to +/-1.25 */
      if (v<-1.25) v=-1.25;
      }
   if (v<0) neg=-1; else neg=0;         /* determine if negative */
   v=sqrt(v*v);
   if (r!=0) d=(v/2.5)*128.0;            /* calculate byte to send to DAC */
      else d=(v/1.25)*128.0;
   if (d>127) d=127;                     /* clip value */
   if (neg!=0) b=BinaryToTwos(d);      /* convert to 2's complement */
      else {
         b=d;
         b=(b & 0x7f);                  /* positive 2's comp number */
      }
   SetByte(b);
}


/*********************************************************************
Description : Searches all ports for the AIU, and sets the global variable
   'PortAddr' to the base address of the parallel port is found. Otherwise
   it sets 'PortAddr' to zero
**********************************************************************/
void DetectAIU(void)
{
   int c, found;
   PortAddr=0x378;
   found=0;                                 /* assume not found */
   for (c=0; c<3; c++) {
      Power(1);                           /* apply power to port */
      SetByte(0xaa);                        /* output test pattern */
      if (ReadByte()==0xaa) {               /* if read back test pattern */
         found=-1;                        /* found AIU */
         break;
      }
      if (found==0) {
         if (PortAddr==0x378) PortAddr=0x278;
            else if (PortAddr==0x278) PortAddr=0x3bc;
      }
   }
   if (found==0) PortAddr=0;
}
