{*************************************************************************
   Analog Interface Unit (AIU) - Driver Routines
   Version 1.0
   Written in Borland Pascal 7.0
   Last edited   : 28/05/95

   Copyright (c) 1994/95 David L. Jones
   Advanced R&D Solutions
   12 Copeland Road
   Lethbridge Park NSW 2770
   Sydney, Australia.
**************************************************************************}
UNIT AIU10;

INTERFACE

FUNCTION  ReadABS:BYTE;
FUNCTION  ReadVoltage(Range:BOOLEAN) : REAL;
FUNCTION  ReadByte:BYTE;
PROCEDURE DetectAIU;
PROCEDURE SetByte(Value : BYTE);
PROCEDURE Power(OnOff:BOOLEAN);
PROCEDURE SetRange(Range:BOOLEAN);
PROCEDURE SetVoltage(Value:REAL; Range:BOOLEAN);

VAR PortAddr:WORD;

IMPLEMENTATION

FUNCTION BinaryToTwos(B:BYTE):BYTE;
{***************************************************************
   DESCRIPTION : Converts B into Two's Complement form
***************************************************************}
BEGIN
   B:=B XOR $FF;
   B:=B AND $7F;
   INC(B);
   B:=B OR 128;
   BinaryToTwos:=B;
END;


FUNCTION TwosToBinary(B:BYTE):BYTE;
{***************************************************************
   DESCRIPTION : Converts B from Two's Complement into Binary
***************************************************************}
BEGIN
   B:=B AND $7F;
   IF B>0 THEN DEC(B);
   B:=B XOR $FF;
   B:=B AND $7F;
   TwosToBinary:=B;
END;


FUNCTION ReadByte:BYTE;
{***************************************************************
   DESCRIPTION : Returns the Current byte on the AIU's data bus.
   PARAMETERS : NONE
***************************************************************}
VAR T:BYTE;                                    {FINAL VALUE}
   V:BYTE;                                     {TEMP PORT VARIABLE}
BEGIN
   T:=0;                                       {RESET FINAL VALUE}
   PORT[PortAddr]:=PORT[PortAddr] AND $FB;   {SELECT "A" DATA - XXXXX0XX }
   V:=PORT[PortAddr+1];                        {GET PORT VALUE}
   V:=V XOR $80;                             {INVERT LAST BIT}
   IF (V AND 128)>0 THEN T:=T+128;
   IF (V AND 64 )>0 THEN T:=T+64;
   IF (V AND 32 )>0 THEN T:=T+1;
   IF (V AND 16 )>0 THEN T:=T+8;
   PORT[PortAddr]:=PORT[PortAddr] OR $4;      {SELECT "B" DATA - XXXXX1XX }
   V:=PORT[PortAddr+1];                        {GET PORT VALUE}
   V:=V XOR $80;                             {INVERT LAST BIT}
   IF (V AND 128)>0 THEN T:=T+32;
   IF (V AND 64 )>0 THEN T:=T+16;
   IF (V AND 32 )>0 THEN T:=T+2;
   IF (V AND 16 )>0 THEN T:=T+4;
   ReadByte:=T;
END;


PROCEDURE ClockData(PortAddr:WORD; Bit : BYTE);
{***************************************************************
   DESCRIPTION : Clocks a Bit into the AIU data register
   PARAMETERS :
      PortAddr - Base port address of current parallel port
      Bit - The bit to be sent
***************************************************************}
BEGIN
   IF BIT>0 THEN PORT[PortAddr]:=PORT[PortAddr] OR 4          {SET BIT 0}
      ELSE PORT[PortAddr]:=PORT[PortAddr] AND $FB;            {CLEAR BIT}
   PORT[PortAddr+2]:=PORT[PortAddr+2] AND $FB;    {LATCH CLK LOW  - XXXXX0XX}
   PORT[PortAddr+2]:=PORT[PortAddr+2] OR 4;       {LATCH CLK HIGH - XXXXX1XX}
END;


PROCEDURE SetByte(Value : BYTE);
{***************************************************************
   DESCRIPTION : Sends "Value" to the DAC
   PARAMETERS :
      Value - The byte to be sent to the DAC
***************************************************************}
VAR TEMP, RETRY : BYTE;
BEGIN
   RETRY:=0;
   REPEAT
      ClockData(PortAddr, Value AND 1);                  {SET BIT 0}
      ClockData(PortAddr, Value AND 16);                  {SET BIT 4}
      ClockData(PortAddr, Value AND 64);                  {SET BIT 6}
      ClockData(PortAddr, Value AND 128);                  {SET BIT 7}
      ClockData(PortAddr, Value AND 2);                  {SET BIT 1}
      ClockData(PortAddr, Value AND 4);                  {SET BIT 2}
      ClockData(PortAddr, Value AND 8);                  {SET BIT 3}
      ClockData(PortAddr, Value AND 32);                  {SET BIT 5}
      INC(RETRY);
   UNTIL (Value=ReadByte) OR (RETRY>100);

   PORT[PortAddr]:=PORT[PortAddr] OR 1;            {SET WR HIGH - XXXXXXX1}
   PORT[PortAddr]:=PORT[PortAddr] AND $FE;         {SET WR LOW  - XXXXXXX0}
   PORT[PortAddr]:=PORT[PortAddr] OR 1;            {SET WR HIGH - XXXXXXX1}
END;


FUNCTION ReadABS:BYTE;
{***************************************************************
   DESCRIPTION :    Reads the actual 8 bit ADC value from the AIU
                  and returns it as a byte.
   NOTE : The AIU returns values in 2's complement form.
   PARAMETERS : NONE
***************************************************************}
VAR LOOP:LONGINT;                              {NUMBER OF WAIT LOOPS}
BEGIN
   PORT[PortAddr]:=PORT[PortAddr] OR 1;      {TURN "WR" OFF     - XXXXXXX1 }
   PORT[PortAddr]:=PORT[PortAddr] AND $FD;   {SET ADC TO "READ" - XXXXXX0X }
   LOOP:=0;
   REPEAT
      INC(LOOP);
   UNTIL ((PORT[PortAddr+1] AND 8)=0) OR (LOOP>100000);
                                             {WAIT UNTIL END OF CONVERSION}
   ReadABS:=ReadByte;
   PORT[PortAddr]:=PORT[PortAddr] OR 2;      {TURN OFF "READ" - XXXXXX1X }
END;


FUNCTION ReadVoltage(Range:BOOLEAN) : REAL;
{***************************************************************
   DESCRIPTION : Returns the current voltage from the AIU
   PARAMETERS :
      Range - TRUE=+/- 2.5V      FALSE=+/- 1.25V
***************************************************************}
VAR B,F:BYTE;
   T:REAL;
   NEGATIVE:BOOLEAN;
BEGIN
   B:=ReadABS;                                             {GET THE ADC VALUE}

   IF (B AND $80)>0 THEN NEGATIVE:=TRUE ELSE NEGATIVE:=FALSE;

   IF NEGATIVE THEN BEGIN
      B:=TwosToBinary(B);
      F:=1;                                                      {FUDGE FACTOR}
      IF Range THEN T:=((B+F)/128)*2.5   ELSE T:=((B+F)/128)*1.25
   END
   ELSE BEGIN
      IF (B=0) OR (B=127) THEN F:=0 ELSE F:=1;                  {FUDGE FACTOR}
      IF Range THEN T:=((B-F)/128)*2.5 ELSE T:=((B-F)/128)*1.25;
   END;

   IF NEGATIVE THEN T:=-T;

   ReadVoltage:=T;
END;


PROCEDURE Power(OnOff:BOOLEAN);
{***************************************************************
   DESCRIPTION : Switches power to the AIU on and off, & initialises port.
   PARAMETERS :
      OnOff - TRUE=Power ON      FALSE=Power OFF
***************************************************************}
BEGIN
   IF OnOff THEN PORT[PortAddr]:=$FF               { 11111111 }
   ELSE PORT[PortAddr]:=0;                           { 00000000 }
   PORT[PortAddr+2]:=0;                     {DISABLE PARALLEL PORT INTERUPTS}
END;


PROCEDURE SetRange(Range:BOOLEAN);
{***************************************************************
   DESCRIPTION : Switches bewteen +/-2.5V and +/-1.25V ranges on the AIU
   PARAMETERS :
      Range - TRUE=+/-2.5V      FALSE=+/-1.25V
***************************************************************}
BEGIN
   IF Range THEN PORT[PortAddr+2]:=PORT[PortAddr+2] AND $FE   { XXXXXXX0 }
      ELSE PORT[PortAddr+2]:=PORT[PortAddr+2] OR 1;         { XXXXXXX1 }
END;


PROCEDURE SetVoltage(Value:REAL; Range:BOOLEAN);
{***************************************************************
   DESCRIPTION : Sets the DAC on the AIU to a specified value.
   PARAMETERS :
      Value - The voltage to be sent to the DAC
      Range - TRUE=+/-2.5v      FALSE=+/-1.25v
***************************************************************}
VAR
   D:WORD;
   B:BYTE;
   NEGATIVE:BOOLEAN;
BEGIN
   IF RANGE THEN BEGIN
      IF VALUE>2.5 THEN VALUE:=2.5;
      IF VALUE<-2.5 THEN VALUE:=-2.5;
   END
   ELSE BEGIN
      IF VALUE>1.25 THEN VALUE:=1.25;
      IF VALUE<-1.25 THEN VALUE:=-1.25;
   END;

   IF VALUE<0 THEN NEGATIVE:=TRUE ELSE NEGATIVE:=FALSE;

   IF Range THEN D:=TRUNC((ABS(Value)/2.5)*128)
      ELSE D:=TRUNC((ABS(Value)/1.25)*128);

   IF D>127 THEN D:=127;

   IF NEGATIVE THEN B:=BinaryToTwos(D)
   ELSE BEGIN
      B:=D;
      B:=B AND $7F;
   END;

   SETBYTE(B);
END;


PROCEDURE DetectAIU;
{***************************************************************
   DESCRIPTION : Searches all ports for the AIU, and sets the global
   variable 'PortAdrr' to the base address of the parallel port address
   if found. Otherwise 'PortAddr' = 0
***************************************************************}
VAR C:WORD;
   FOUND:BOOLEAN;
BEGIN
   FOUND:=FALSE;                                       {ASSUME NOT FOUND}
   PortAddr:=$378;

   FOR C:=1 TO 3 DO BEGIN
      Power(TRUE);                                    {POWER UP AIU ON PORT}
      SetByte($AA);
      IF ReadByte=$AA THEN FOUND:=TRUE;               {FOUND AIU ON PORT}

      IF NOT FOUND THEN
         IF PortAddr=$378 THEN PortAddr:=$278
            ELSE IF PortAddr=$278 THEN PortAddr:=$3BC;
   END;

   IF NOT FOUND THEN PortAddr:=0;
END;
END.
