DECLARE SUB SETVOLTAGE (Value!, Range%)
DECLARE FUNCTION BinaryToTwos% (B%)
DECLARE SUB ClockData (BIT%)
DECLARE SUB POWER (OnOff%)
DECLARE FUNCTION ReadABS% ()
DECLARE FUNCTION ReadByte% ()
DECLARE FUNCTION ReadVoltage! (Range%)
DECLARE SUB SetByte (Value%)
DECLARE FUNCTION TwosToBinary% (B%)
DECLARE SUB DetectAIU ()
DECLARE SUB SETRANGE (Range%)
'*****************************************************************
' Analog Interface Unit (AIU) - Driver Routines
' Version 1.0
' Written for MSDOS QBASIC
' Last Edited : 28/5/95

' Copyright (C) 1995 David L. Jones
' Advanced R&D Solutions
' 12 Copeland Road
' Lethbridge Park NSW 2770
' Sydney, Australia.
'*****************************************************************
DEFINT A-Z

DIM SHARED PortAddr%

CLS

CALL DetectAIU

CALL POWER(1)
CALL SETRANGE(1)

CLS

FOR C! = -2.5 TO 2.5 STEP 5 / 256
   CALL SETVOLTAGE(C!, 1)
   D! = ReadVoltage!(1)
   PRINT USING "##.###"; C!;
   PRINT "     ";
   PRINT USING "##.###"; D!
   FOR Z = 1 TO 32000: NEXT Z
NEXT C!

CLS

CALL SETRANGE(0)

FOR C! = -1.25 TO 1.25 STEP 2.5 / 256
   CALL SETVOLTAGE(C!, 0)
   D! = ReadVoltage!(0)
   PRINT USING "##.###"; C!;
   PRINT "     ";
   PRINT USING "##.###"; D!
   FOR Z = 1 TO 32000: NEXT Z
NEXT C!

FUNCTION BinaryToTwos% (B%)

'*********************************************************************
'DESCRIPTION: CONVERTS B% INTO TWO'2 COMPLEMENT FORM
'*********************************************************************

B% = B% XOR &HFF
B% = B% AND &H7F
IF B% < 127 THEN B% = B% + 1
B% = B% OR 128
BinaryToTwos% = B%

END FUNCTION

SUB ClockData (BIT%)

'*********************************************************************
'DESCRIPTION: CLOCKS DATA INTO THE AIU
'PARAMETERS:
'  BIT% - 0=FALSE    <>0=TRUE
'*********************************************************************

IF BIT% > 0 THEN
   OUT PortAddr%, INP(PortAddr%) OR 4           'SET BIT 0
ELSE
   OUT PortAddr%, INP(PortAddr%) AND &HFB       'CLEAR BIT
END IF

OUT PortAddr% + 2, INP(PortAddr% + 2) AND &HFB  'LATCH CLOCK LOW  - XXXXX0XX
OUT PortAddr% + 2, INP(PortAddr% + 2) OR 4      'LATCH CLOCK HIGH - XXXXX1XX

END SUB

SUB DetectAIU

'*********************************************************************
'DESCRIPTION: SEARCHES ALL PARALLEL PORTS FOR THE AIU, AND SET THE GLOBAL
'              VARIABLE 'PortAddr' TO THE BASE ADDRESS IF FOUND. OTHERWISE
'              'PortAddr' IS SET TO ZERO
'*********************************************************************
                                                      
DIM FOUND%, C%

FOUND% = 0                                               'ASSUME NOT FOUND
PortAddr% = &H378

FOR C% = 1 TO 3
   CALL POWER(-1)                             'POWER UP AIU ON PORT
   CALL SetByte(0)
   IF ReadByte = 0 THEN                       'TEST FOR ZERO
      CALL SetByte(&HAA)                      'TEST FOR
      IF ReadByte = &HAA THEN
         FOUND = -1                                      'FOUND AIU, EXIT
         EXIT FOR
      END IF
   END IF

   IF FOUND% = 0 THEN
      IF PortAddr% = &H378 THEN
         PortAddr% = &H278
      ELSE
         IF PortAddr% = &H278 THEN PortAddr% = &H3BC
      END IF
   END IF
NEXT C%

IF FOUND = 0 THEN PortAddr% = 0

END SUB

SUB POWER (OnOff%)

'*********************************************************************
'DESCRIPTION: TURNS POWER TO THE AIU ON AND OFF
'PARAMETERS:
'  OnOff% - IF =0 THEN TURN POWER OFF, ELSE TURN POWER ON
'*********************************************************************

IF (OnOff <> 0) THEN OUT PortAddr%, &HFF ELSE OUT PortAddr%, 0

OUT PortAddr% + 2, 0                      '* DISABLE PORT INTERUPTS

END SUB

FUNCTION ReadABS%

'*********************************************************************
'DESCRIPTION: READS THE ACTUAL 8 BIT ADC VALUE & RETURNS IT AS A BYTE
'PARAMETERS: NONE
'*********************************************************************

DIM LOOPS%                             'NUMBER OF WAIT LOOPS

OUT PortAddr%, INP(PortAddr%) OR 1     'TURN "WR" OFF     - XXXXXXX1
OUT PortAddr%, INP(PortAddr%) AND &HFD 'SET ADC TO "READ" - XXXXXX0X
LOOPS% = 0
DO
   LOOPS% = LOOPS% + 1
LOOP UNTIL ((INP(PortAddr% + 1) AND 8) = 0) OR (LOOPS% > 100000)
                                       'WAIT UNTIL END OF CONVERSION
ReadABS% = ReadByte
OUT PortAddr%, INP(PortAddr%) OR 2     'TURN OFF "READ" - XXXXXX1X

END FUNCTION

FUNCTION ReadByte%

'*********************************************************************
'DESCRIPTION: RETURNS THE CURRENT BYTE ON THE AIU'S DATA BUS
'PARAMETERS: NONE
'*********************************************************************

DIM T%                                 'FINAL VALUE
DIM V%                                 'TEMP PORT VARIABLE

T% = 0                                 'RESET FINAL VALUE

OUT PortAddr%, INP(PortAddr%) AND &HFB 'SELECT "A" DATA - XXXXX0XX
V% = INP(PortAddr% + 1)                'GET PORT VALUE
V% = V% XOR &H80                       'INVERT LAST BIT
IF (V% AND 128) > 0 THEN T% = T% + 128
IF (V% AND 64) > 0 THEN T% = T% + 64
IF (V% AND 32) > 0 THEN T% = T% + 1
IF (V% AND 16) > 0 THEN T% = T% + 8

OUT PortAddr%, INP(PortAddr%) OR &H4   'SELECT "B" DATA - XXXXX1XX
V% = INP(PortAddr% + 1)                'GET PORT VALUE
V% = V% XOR &H80                       'INVERT LAST BIT
IF (V% AND 128) > 0 THEN T% = T% + 32
IF (V% AND 64) > 0 THEN T% = T% + 16
IF (V% AND 32) > 0 THEN T% = T% + 2
IF (V% AND 16) > 0 THEN T% = T% + 4

ReadByte% = T%

END FUNCTION

FUNCTION ReadVoltage! (Range%)

'*********************************************************************
'DESCRIPTION: RETURNS THE CURRENT VOLTAGE FROM THE ADC
'PARAMETERS:
'  Range% - 0=+/-1.25V  <>0=+/-2.5V
'*********************************************************************

DIM B%, F%, T!, NEG%

B% = ReadABS                         '* GET THE ADC VALUE

IF (B% AND &H80) > 0 THEN NEG% = -1 ELSE NEG% = 0
IF NEG% THEN B% = TwosToBinary%(B%)

IF NEG% THEN
   F% = 1
   IF Range% THEN
      T! = ((B% + F%) / 128) * 2.5
   ELSE
      T! = ((B% + F%) / 128) * 1.25
   END IF
ELSE
   IF (B% = 0) OR (B% = 127) THEN F% = 0 ELSE F% = 1      '* FUDGE FACTOR
   IF Range% THEN
      T! = ((B% - F%) / 128) * 2.5
   ELSE
      T! = ((B% - F%) / 128) * 1.25
   END IF
END IF

IF NEG% THEN T! = -T!

ReadVoltage! = T!

END FUNCTION

SUB SetByte (Value%)

'*********************************************************************
'DESCRIPTION: OUTPUTS A BYTE TO THE DAC
'PARAMETERS:
'  Value% - BYTE TO BE OUTPUTED TO THE DAC
'*********************************************************************

DIM TEMP%, RETRY%

DO
   OUT PortAddr% + 2, INP(PortAddr% + 2) AND &HFB  'LATCH CLOCK LOW - XXXXX0XX
   CALL ClockData(Value% AND 1)         'SET BIT 0
   CALL ClockData(Value% AND 16)        'SET BIT 4
   CALL ClockData(Value% AND 64)        'SET BIT 6
   CALL ClockData(Value% AND 128)       'SET BIT 7
   CALL ClockData(Value% AND 2)         'SET BIT 1
   CALL ClockData(Value% AND 4)         'SET BIT 2
   CALL ClockData(Value% AND 8)         'SET BIT 3
   CALL ClockData(Value% AND 32)        'SET BIT 5
   RETRY% = RETRY% + 1
LOOP UNTIL (Value% = ReadByte%) OR (RETRY > 100)

OUT PortAddr%, INP(PortAddr%) OR 1                 'SET WR HIGH - XXXXXXX1
OUT PortAddr%, INP(PortAddr%) AND &HFE             'SET WR LOW  - XXXXXXX0
OUT PortAddr%, INP(PortAddr%) OR 1                 'SET WR HIGH - XXXXXXX1

END SUB

SUB SETRANGE (Range%)

'*********************************************************************
'DESCRIPTION: CHANGES THE VOLTAGE RANGE ON THE AIU
'PARAMETERS:
'  Range% - 0=+/-1.25V     <>0=+/-2.5V
'*********************************************************************

IF (Range% <> 0) THEN
   OUT PortAddr% + 2, INP(PortAddr% + 2) AND &HFE
ELSE
   OUT PortAddr% + 2, INP(PortAddr% + 2) OR 1
END IF

END SUB

SUB SETVOLTAGE (Value!, Range%)

'*********************************************************************
'DESCRIPTION: SETS THE VOLTAGE ON THE DAC
'PARAMETERS:
'  Value! - VOLTAGE TO BE OUTPUTED TO THE DAC
'  Range% - 0=+/-1.25V     <>0=+/-2.5V
'*********************************************************************

DIM NEG%, D%, B%

IF (Range% <> 0) THEN
   IF Value! > 2.5 THEN Value! = 2.5
   IF Value! < -2.5 THEN Value! = -2.5
ELSE
   IF Value! > 1.25 THEN Value! = 1.25
   IF Value! < -1.25 THEN Value! = -1.25
END IF

IF Value! < 0 THEN NEG% = -1 ELSE NEG% = 0

IF Range% THEN
   D% = CINT((ABS(Value!) / 2.5) * 128)
ELSE
   D% = CINT((ABS(Value!) / 1.25) * 128)
END IF

IF D% > 127 THEN D% = 127

IF NEG% THEN
   B% = BinaryToTwos(D%)
ELSE
   B% = D%
   B% = B% AND &H7F
END IF

CALL SetByte(B%)

END SUB

FUNCTION TwosToBinary% (B%)

'*********************************************************************
'DESCRIPTION: CONVERTS B% FROM TWO'S COMPLEMENT INTO BINARY
'*********************************************************************

B% = B% AND &H7F
IF B% > 0 THEN B% = B% - 1
B% = B% XOR &HFF
B% = B% AND &H7F

TwosToBinary% = B%

END FUNCTION

