{*************************************************************************
   Analog Interface Unit (AIU) - VoltMeter - Version 1.0
   Written in Borland Pascal 7.0
   Last edited   : 20/5/95

   Copyright (c) 1994/95 David L. Jones
   Advanced R&D Solutions - TRONNORT Technology
   12 Copeland Road
   Lethbridge Park NSW 2770
   Sydney, Australia.
***************************************************************************}

PROGRAM AIU_VOLTMETER;
USES CRT, DOS, AIU10;

CONST
   MAX_SAMPLES=250;                           {MAXIMUM NUMBER OF SAMPLES}
VAR
   VALUE, MIN, MAX, AVERAGE, RMS : REAL;     {CURRENT DISPLAY VALUES}
   MINTIME, MAXTIME:DATETIME;                  {TIME STAMP FOR MIN/MAX VALUES}
   RANGE:BOOLEAN;                              {TRUE=2.5V   FALSE=1.25V}
   TEMP:WORD;
   READINGS:ARRAY[0..MAX_SAMPLES] OF REAL;   {ARRAY OF MESUREMENTS FOR CALC.}

{******************************************************}
PROCEDURE WRSTR(X, Y: INTEGER; S: STRING);
BEGIN
  GOTOXY(X, Y);
  WRITE(S);
END;

PROCEDURE DISPLAY_BOX(XSTART, YSTART, WIDTH, DEPTH : BYTE);
{***************************************************************
DISPLAYS A DOUBLE LINE BOX IN THE CURRENT ATTRIBUTES.
PARAMETERS :
   XSTART, YSTART - TOP LEFT CORNER OF BOX OUTLINE
   WIDTH - NUMBER OF CHARACTERS WIDE INSIDE BOX
   DEPTH - NUMBER OF LINES OF INSIDE OF BOX
****************************************************************}
VAR C:BYTE;
   S1,S2:STRING;
BEGIN
   S1:=''; S2:='';
   FOR C:=1 TO WIDTH DO BEGIN S1:=S1+''; S2:=S2+' '; END;
   WRSTR(XSTART,YSTART, ''+S1+'');
   FOR C:=YSTART+1 TO YSTART+DEPTH DO WRSTR(XSTART, C, ''+S2+'');
   WRSTR(XSTART, YSTART+DEPTH+1, ''+S1+'');
END;

{***************************************************************
CALCULATES THE AVERAGE AND RMS VALUES FOR THE VALUES IN THE "READINGS" ARRAY.
****************************************************************}
PROCEDURE CALCULATE_VALUES;
VAR C:WORD;
   TOTAL, TOTALSQR:REAL;
BEGIN
   AVERAGE:=0;    RMS:=0;
   TOTALSQR:=0; TOTAL:=0;
   FOR C:=0 TO MAX_SAMPLES DO BEGIN
      TOTAL:=TOTAL+READINGS[C];
      TOTALSQR:=TOTALSQR+SQR(READINGS[C]);
   END;
   AVERAGE:=TOTAL/(MAX_SAMPLES+1);
   RMS:=SQRT(TOTALSQR/(MAX_SAMPLES+1));
END;

{***************************************************************}
PROCEDURE DISPLAY_MAINSCREEN;
BEGIN
   TEXTCOLOR(LIGHTGREEN); TEXTBACKGROUND(BLACK);
   CLRSCR;
   WRSTR(10,1, 'AIU VoltMeter - Ver1.0');
   TEXTCOLOR(LIGHTGRAY);
   WRSTR(1,4, 'Actual  Value:');
   WRSTR(1,7, 'Average Value:');
   WRSTR(1,10,'TrueRMS Value:');
   WRSTR(1,13,'Maximum Value:');
   WRSTR(1,16,'Minimum Value:');
   TEXTCOLOR(WHITE);
   DISPLAY_BOX(15, 3,10,1);
   DISPLAY_BOX(15, 6,10,1);
   DISPLAY_BOX(15, 9,10,1);
   DISPLAY_BOX(15,12,10,1);
   DISPLAY_BOX(27,12,10,1);
   DISPLAY_BOX(15,15,10,1);
   DISPLAY_BOX(27,15,10,1);
   TEXTCOLOR(WHITE);
   WRSTR(8,21, '(V)oltage range - ');
   TEXTCOLOR(LIGHTRED);
   IF RANGE THEN WRITE('+/- 2.5V ') ELSE WRITE('+/- 1.25V');
   TEXTCOLOR(WHITE);
   WRSTR(8,22, '(R)eset min/max');
   WRSTR(16,24, 'ESC-Quit');
END;

{***************************************************************}
PROCEDURE DISPLAY_VALUES;
VAR C : BYTE;
BEGIN
   TEXTCOLOR(LIGHTRED);
   GOTOXY(18,4); WRITE(ABS(VALUE):1:2,' V');
   IF VALUE<0 THEN WRSTR(17,4,'-') ELSE WRSTR(17,4,' ');
   GOTOXY(18,7); WRITE(ABS(AVERAGE):1:2,' V');
   IF AVERAGE<0 THEN WRSTR(17,7,'-') ELSE WRSTR(17,7,' ');
   GOTOXY(18,10); WRITE(ABS(RMS):1:2,' V');
   GOTOXY(18,13);   WRITE(ABS(MAX):1:2,' V');
   IF MAX<0 THEN WRSTR(17,13,'-') ELSE WRSTR(17,13,' ');
   GOTOXY(18,16);   WRITE(ABS(MIN):1:2,' V');
   IF MIN<0 THEN WRSTR(17,16,'-') ELSE WRSTR(17,16,' ');
   GOTOXY(29,13); WRITE(MAXTIME.HOUR:2,':',MAXTIME.MIN:2,':',MAXTIME.SEC:2);
   GOTOXY(29,16); WRITE(MINTIME.HOUR:2,':',MINTIME.MIN:2,':',MINTIME.SEC:2);
END;

PROCEDURE MAIN_ROUTINE;
{***************************************************************
MAIN PROCESSING LOOP. CONTINUOUSLY PROCESSES AND DISPLAYS DATA.
****************************************************************}
VAR
   NUM,TEMP:WORD;                              {NUMBER OF SAMPLES TAKEN}
   KEY:CHAR;
BEGIN
   DETECTAIU;
   POWER(TRUE);
   RANGE:=TRUE;
   VALUE:=READVOLTAGE(RANGE);
   MIN:=VALUE;
   MAX:=VALUE;
   GETTIME(MAXTIME.HOUR, MAXTIME.MIN, MAXTIME.SEC, TEMP);
   GETTIME(MINTIME.HOUR, MINTIME.MIN, MINTIME.SEC, TEMP);
   DISPLAY_MAINSCREEN;
   SETRANGE(RANGE);
   NUM:=0;                                          {NUMBER OF SAMPLES TAKEN}

   REPEAT
      VALUE:=READVOLTAGE(RANGE);
      READINGS[NUM]:=VALUE;                        {STORE SAMPLE VALUE}
      DISPLAY_VALUES;                              {UPDATE ALL VALUES}

      IF VALUE>MAX THEN BEGIN                        {NEW MAXIMUM VALUE}
         MAX:=VALUE;
         GETTIME(MAXTIME.HOUR, MAXTIME.MIN, MAXTIME.SEC, TEMP);
      END;

      IF VALUE<MIN THEN BEGIN                        {NEW MINIMUM VALUE}
         MIN:=VALUE;
         GETTIME(MINTIME.HOUR, MINTIME.MIN, MINTIME.SEC, TEMP);
      END;

      IF NUM>=MAX_SAMPLES THEN BEGIN               {MAX NUM OF SAMPLES TAKEN}
         NUM:=0;                                    {RESET SAMPLES COUNTER}
         CALCULATE_VALUES;                           {CALC. AVERAGE & RMS}
      END
      ELSE INC(NUM);

      IF KEYPRESSED THEN BEGIN
         KEY:=UPCASE(READKEY);
         IF KEY='V' THEN BEGIN
            RANGE:=NOT RANGE;
            SETRANGE(RANGE);
         END;
         IF (KEY='R') OR (KEY='V') THEN BEGIN
            MAX:=VALUE;
            GETTIME(MAXTIME.HOUR, MAXTIME.MIN, MAXTIME.SEC, TEMP);
            MIN:=VALUE;
            GETTIME(MINTIME.HOUR, MINTIME.MIN, MINTIME.SEC, TEMP);
            DISPLAY_MAINSCREEN;
         END;
      END;
   UNTIL KEY=#27;
   POWER(FALSE);
END;

{********************************************************}
PROCEDURE HIDE_CURSOR;
VAR REG:REGISTERS;
BEGIN
   REG.CH:=32;                      {MAGIC VALUE TO TURN OFF THE CURSOR}
   REG.AH:=1;                       {"SET CURSOR SIZE" DOS FUNCTION}
   INTR($10,REG);                   {CALL INTERRUPT 10H}
END;

{********************************************************}
PROCEDURE RESTORE_CURSOR;
VAR REG:REGISTERS;
BEGIN
   REG.CH:=11;                      {RESIZE THE CURSOR}
   REG.CL:=12;
   REG.AH:=1;                       {"SET CURSOR SIZE" DOS FUNCTION}
   INTR($10,REG);                   {CALL INTERRUPT 10H}
END;

{********************************************************}
BEGIN
   TEXTMODE(CO40);
   HIDE_CURSOR;
   MAIN_ROUTINE;
   TEXTMODE(CO80);
   RESTORE_CURSOR;
   CLRSCR;
END.
